# Migration Guides

## Migrate `flutter_custom_tabs` from v1.x to v2.0.0
The first major update of `flutter_custom_tabs`, v2.0.0, includes a variety of new features and improvements, as well as some breaking changes.

### Breaking Changes

#### API Change: `launch` to `launchUrl`
The `launch` method has been renamed to `launchUrl` for better clarity and consistency. Additionally, there are changes in the naming and types of some parameters:

```diff
-CustomTabsOption? customTabsOption,
+CustomTabsOptions? customTabsOptions,
-SafariViewControllerOption? safariVCOption,
+safariViewControllerOptions? safariVCOptions,
```

New signature:
```dart
// Before:
Future<void> launch(
  String urlString, {
  CustomTabsOption? customTabsOption,
  SafariViewControllerOption? safariVCOption,
}) async;

// After:
Future<void> launchUrl(
  Uri url, {
  bool prefersDeepLink,
  CustomTabsOptions? customTabsOptions,
  SafariViewControllerOptions? safariVCOptions,
}) async;
```

Related changes:
- The method `closeAllIfPossible` has been renamed to `closeCustomTabs`.

#### API Changes in `CustomTabsOptions`

The `CustomTabsOptions` class has undergone several changes:
- The customization of various colors, including the toolbar color, has been moved to the `CustomTabsColorSchemes` class.

```dart
// Before:
/// Custom tab toolbar color.
final Color? toolbarColor;

// After:
/// The visualization configuration.
final CustomTabsColorSchemes? colorSchemes;
```

To migrate, use the following equivalent options:
```dart
// Before
CustomTabsOption(
  toolbarColor: Colors.blue,
)

// After
CustomTabsOptions(
  colorSchemes: CustomTabsColorSchemes.defaults(
      toolbarColor: Colors.blue,
      // and newly added the system navigation colors.
      // navigationBarColor: any color
      // navigationBarDividerColor any color
  ),
)
```

- The method of specifying the share state has changed from `enableDefaultShare` to `shareState`:

| Before | After |
| --- | --- |
| `enableDefaultShare: true` | `shareState: CustomTabsShareState.on` |
| `enableDefaultShare: false` | `shareState: CustomTabsShareState.off` |


- The method of specifying the close button has been changed from `closeButtonPosition` to the enhanced `closeButton`.

To migrate, use the following equivalent options:
```dart
// Before:
closeButtonPosition: CustomTabsCloseButtonPosition.end

// After:
closeButton: CustomTabsCloseButton(
  position: CustomTabsCloseButtonPosition.end,
  // and newly added the button icon.
  // icon: CustomTabsCloseButtonIcon.back,
  // or
  // icon: "DRAWABLE_RESOURCE_ID_IN_YOUR_ANDROID_PROJECT",
)
```

Remaining name changes:

| Change Type | Before | After |
| --- | --- | --- |
| Property | `CustomTabsOption.showPageTitle` | `CustomTabsOptions.showTitle` |
| Property | `CustomTabsOption.enableInstantApps` | `CustomTabsOptions.instantAppsEnabled` |
| Class | `CustomTabsAnimation` | `CustomTabsAnimations` |
| Class | `CustomTabsSystemAnimation` | `CustomTabsSystemAnimations` |
| Property | `CustomTabsOption.animation` | `CustomTabsOptions.animations` |

#### API Changes in `safariViewControllerOptions`

- The property `statusBarBrightness` has been deleted.

#### Lightweight `flutter_custom_tabs`

> **Note**
> This is an experimental feature in the pre-release phase of v2.0.0.

`flutter_custom_tabs` provides a rich set of customization options for Custom Tabs, but sometimes a minimal appearance customization is enough.  
The newly introduced `LaunchOptions` provides unified and simple options for Android/iOS.

Start by importing the library file:
```diff
-import 'package:flutter_custom_tabs/flutter_custom_tabs.dart';
+import 'package:flutter_custom_tabs/flutter_custom_tabs_lite.dart';
```

Finally, use `LaunchOptions` to launch a URL.  
(The following URL launch results in the same outcome):

```dart
final theme = ...;
// Before:
await launch(
  'https://flutter.dev',
  customTabsOptions: CustomTabsOptions(
    toolbarColor: theme.colorScheme.surface,
    enableDefaultShare: true,
    enableUrlBarHiding: true,
    showTitle: true,
  ),                    
  safariVCOptions: SafariViewControllerOptions(
    preferredBarTintColor: theme.colorScheme.surface,
    preferredControlTintColor: theme.colorScheme.onSurface,
    barCollapsingEnabled: true,
  ),
);

// After:
await launchUrl(
  Uri.parse('https://flutter.dev'),
  options: LaunchOptions(
    barColor: theme.colorScheme.surface,
    onBarColor: theme.colorScheme.onSurface,
    appBarFixed: false,
  ),
);
```

## Migrate `flutter_custom_tabs` to v1.0.0

flutter_custom_tabs v1.0.0 is the first major release and has been updated to include several breaking changes.
- Migrate to federated plugins
- Improved customization on iOS
- Web support

### API Changes
* `launch` function now accepts two options.
  - `customTabsOption`: for Android(optional)
  - `safariVCOption`: for iOS(optional)

```diff
await launch(
    'https://flutter.dev',
-    option
+    customTabsOption: CustomTabsOption(),
+    safariVCOption: SafariViewControllerOption(),
);
```  

* The built-in animation for CustomTabs `CustomTabsAnimation` is now `CustomTabsSystemAnimation`
  - This clarifies the use of OS-supplied animations via plug-ins.

```diff
customTabsOption: CustomTabsOption(
-    animation: CustomTabsAnimation.slideIn(),
+    animation: CustomTabsSystemAnimation.slideIn(),
)
```
